#include <map>
#include <string>

#include <X11/Xlib.h>

////////////////////////////////////////////////////////////////////////////////
class App
   {
   public:

   /////////////////////////////////////
   class BitMap
      {
      public:
      BitMap(const std::string & file);
      ~BitMap();

      void make_pixmap(Display * dpy, Drawable d);

      // Yeah, it's all public here.  Keeping it simple.
      std::string name;
      unsigned int width;
      unsigned int height;
      unsigned char * data;
      Pixmap pixmap;

      private:
      BitMap(const App::BitMap & b);

      int read_from_file(const std::string & file);
      };

   /////////////////////////////////////
   // TODO: Need to be able to support child windows.
   class WindowData
      {
      public:
      WindowData(const std::string & n, Window w, App::BitMap * i) :
	 name(n), win(w), icon(i) {};
      ~WindowData()
	 {
	 if (icon) { delete icon; icon = NULL; }
	 }


      std::string name;
      Window win;
      App::BitMap * icon;
      };

   /////////////////////////////////////

   typedef std::pair<std::string, App::WindowData *> win_pair;
   typedef std::map<std::string, App::WindowData *> WindowMap;

   App(const std::string & appname);
   ~App();

   static int x_error_handler(Display * dpy, XErrorEvent *err);

   void create_window(const std::string& win_name,
		      int x, int y,
		      unsigned int width, unsigned int height,
		      App::BitMap * icon);

   inline int display_height(void) const
      { return DisplayHeight(dpy, DefaultScreen(dpy)); };

   inline int display_width(void) const
      { return DisplayWidth(dpy, DefaultScreen(dpy)); };

   void run(void);

   private:
   App();
   App(const App & a);

   Display * dpy;
   WindowMap windows;
   Atom wm_delete_window;  // Need this in contructor and run().

   };
